@@ -11,5 +11,6 @@ class BasicRequest(object):
     def __init__(self):
         self.logger = logging.getLogger("RotatingLog")
 
-    def to_uri(self):
-        return '{0}://{1}:{2}{3}'.format(self.protocol, self.host, self.port, self.endpoint)
+    def to_uri(self, endpoint=None):
+        endpoint = endpoint or self.endpoint
+        return '{0}://{1}:{2}{3}'.format(self.protocol, self.host, self.port, endpoint)
